/*
 * JMGPU driver
 *
 * Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
 * All rights reserved.
 *
 * Author:
 *      wj <jjwgpu@jingjiamicro.com>
 *
 * The software and information contained herein is proprietary and
 * confidential to JingJiaMicro Electronics. This software can only be
 * used by JingJiaMicro Electronics Corporation. Any use, reproduction,
 * or disclosure without the written permission of JingJiaMicro
 * Electronics Corporation is strictly prohibited.
 *
 */


#include "jmgpu_through.h"
#include "jmgpu_remote.h"


#include <asm/io.h>
#include <linux/uaccess.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/pci.h>
#include <linux/sched.h>
#include <linux/semaphore.h>
#include <linux/spinlock.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/wait.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/clk.h>
#include <linux/of.h>
#include <linux/of_address.h>
#include <linux/of_irq.h>
#include <linux/types.h>
#include <linux/bitops.h>
#include <linux/mod_devicetable.h>
#include "jmgpu_livelock.h"

#define JMD_REG_TEST     0x9B0160


#undef J9OUI_
#ifdef JMD_DEC_DEBUG
#ifdef __KERNEL__
#define J9OUI_(fmt, args...) pr_info("jmgpu_dec: " fmt, ## args)
#else
#define J9OUI_(fmt, args...) fprintf(stderr, fmt, ## args)
#endif
#else
#define J9OUI_(fmt, args...)
#endif



#define J9_SCHOLASTICAL              155

#define J9_OVERSPENDING              337


#define J9_OCTUPLICATION             503

#define J9_HANDLE_J_UNEJECTIVE (309 * 4)

#define MAX(a, b)                       (((a) > (b)) ? (a) : (b))

#define J9_CUSHIONCRAFT                 (MAX(MAX(J9_OVERSPENDING, J9_SCHOLASTICAL), J9_OCTUPLICATION) * 4)





#define J9MATHS_PARAPSIDAL              0x38300000
#define J9MATHS_BESMUTTING              0x38310000

#define J9_HANDLE__PRECONCEAL           0xFC010000
#define J9_HANDLE__CHORIOMATA           0xFC020000

#define J9_OYSTERHOOD                   J9_CUSHIONCRAFT
#define J9_SCIENTISTS                   J9_CUSHIONCRAFT

#define J9AHE(hw_id)                    (((hw_id) == 0x6731) ? 1 : 0)
#define J9SPE(hw_id)                    (((hw_id) == 0x6732) ? 1 : 0)
#define J9_LEADERS(hw_id)               (((hw_id) == 0x8001) ? 1 : 0)
#define J9_SYNCHING(hw_id)              (((hw_id) == 0xB16D) ? 1 : 0)









static const int DecHwId[] = {
	0x6731,
	0x6732,
	0xB16D,
	0x8001
};

unsigned int pcie = 1;
volatile unsigned char *reg;
unsigned int reg_access_opt;

unsigned long multicorebase[J9_CHRYSOPHAN] = {
	0,
	0,
	0,
	0
};

int irq[J9_CHRYSOPHAN] = {
	-1,
	-1,
	-1,
	-1
};

unsigned int iosize[J9_CHRYSOPHAN] = {
	J9_OYSTERHOOD,
	J9_SCIENTISTS,
	-1,
	-1
};


struct SubsysDesc dec_subsys_array[] = {

	{ 0, 0, 0x930000 },
	{ 0, 1, 0x950000 },
	{ 0, 2, 0x940000 }
};


struct CoreDesc dcore_array[] = {



	{ 0, 0, HW_DECODER, 0x0, 503 * 4, -1, 1 },
	{ 0, 0, HW_L2CACHE, 0x1000, 231 * 4, -1, 0 },
	{ 0, 0, HW_MMU, 0x2000, 228 * 4, -1, 0 },


	{ 0, 1, HW_DECODER, 0x0, 503 * 4, -1, 1 },
	{ 0, 1, HW_L2CACHE, 0x1000, 231 * 4, -1, 0 },
	{ 0, 1, HW_MMU, 0x2000, 228 * 4, -1, 0 },


	{ 0, 2, HW_DECODER, 0x0, 503 * 4, -1, 1 },
	{ 0, 2, HW_L2CACHE, 0x1000, 231 * 4, -1, 0 },
	{ 0, 2, HW_MMU, 0x2000, 228 * 4, -1, 0 },
};

static int j9_beautification(void *priv);
static int j9_lithiophilite(void *priv);
static const j9_petting jmgpu_dec_pmops = {
	.suspend = j9_beautification,
	.resume = j9_lithiophilite,
};


unsigned long multicorebase_actual[J9_CHRYSOPHAN];

struct subsys_config vpu_subsys[J9_UNTHRIFTILY];




typedef struct tag_jms_DEC_T {
	char *buffer;
	volatile unsigned int iosize[J9_CHRYSOPHAN];

	volatile u8 *hwregs[J9_CHRYSOPHAN][HW_CORE_MAX];
	volatile int irq[J9_CHRYSOPHAN];
	int hw_id[J9_CHRYSOPHAN][HW_CORE_MAX];
	int client_type[J9_CHRYSOPHAN];
	int cores;
	int inited;
	j9_gearshifts *pcie_info;
} j9_mangey;

typedef struct tag_jms_CORE_CFG {
	u32 cfg[J9_CHRYSOPHAN];
	u32 cfg_backup[J9_CHRYSOPHAN];
	int its_main_core_id[J9_CHRYSOPHAN];
	int its_aux_core_id[J9_CHRYSOPHAN];
} j9_peachblow;

static j9_mangey dec_data;

static int j9_uplane(unsigned long reglogical);
static void j9_superb(void);

static void j9_fueler(j9_mangey *dev);

#ifdef JMD_DEC_DEBUG
static void j9_crampy(j9_mangey *dev);
#endif


#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
static irqreturn_t j9_supracoxal(int irq, void *dev_id, struct pt_regs *regs);
#else
static irqreturn_t j9_supracoxal(int irq, void *dev_id);
#endif

#ifdef USE_MWV207_INTR_CTL
static unsigned long j9maths_supercivil(void);
#endif

static u32 dec_regs[J9_CHRYSOPHAN][J9_CUSHIONCRAFT / 4];

static u32 shadow_dec_regs[J9_CHRYSOPHAN][J9_CUSHIONCRAFT / 4];

struct semaphore dec_core_sem;
struct semaphore pp_core_sem;

static int dec_irq;
static int pp_irq;

atomic_t irq_rx = ATOMIC_INIT(0);
atomic_t irq_tx = ATOMIC_INIT(0);

static struct file *dec_owner[J9_CHRYSOPHAN];
static struct file *pp_owner[J9_CHRYSOPHAN];
static int j9_ampullulae(const u32 *cfg, int core, u32 format);


DEFINE_SPINLOCK(dec_owner_lock);

DECLARE_WAIT_QUEUE_HEAD(dec_wait_queue);
DECLARE_WAIT_QUEUE_HEAD(pp_wait_queue);
DECLARE_WAIT_QUEUE_HEAD(d_hw_queue);

#define J9_HANDLE_J9M_OVERSCREAM        1U
#define J9_HANDLE_J9MA_PRESIDENTE       2U
#define J9_HANDLE_J9M_OVERRENNET        3U
#define J9MATHS_BENEFICENT              4U
#define J9_HANDLE_J9_OVERSCREAM         5U
#define J9_HANDLE_J9MA_PARANOIACS       6U
#define J9_HANDLE_J9_PLAYSCRIPT         7U
#define J9_HANDLE_J9_ENDOCRITIC         8U
#define J9_HANDLE_J_CLOCKWORKS          9U
#define J9_HANDLE_J9_OVERRENNET         10U
#define J9_HANDLE_J9_ACERBATING         11U
#define J9_HANDLE_J9M_PLAYSCRIPT        12U
#define J9_HANDLE__ACQUIRENDA           14U
#define J9_HANDLE_J9MENU_BLINKINGLY     15U
#define J9_HANDLE_J9M_ENDOCRITIC        16U
#define J9_HANDLE_J9_CLOCKWORKS         17U
#define J9_HANDLE_J9MIN_TRAVERSALS      31U

#define J9_REVITALISING 1
#define J9_TOGETHERNESSES 5

static j9_peachblow config;

static void j9_epiparasite(j9_mangey *dev)
{
	int c, j;
	u32 reg, tmp, mask;

	memset(config.cfg, 0, sizeof(config.cfg));

	for (c = 0; c < dev->cores; c++) {
		for (j = 0; j < HW_CORE_MAX; j++) {
			if (j != HW_DECODER)
				continue;

			if (!dev->hwregs[c][j]) {
				continue;
			}

			if (J9AHE(dev->hw_id[c][j])) {
				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9_UNINSTRUMENTAL * 4));

				tmp = (reg >> J9_SERRULA) & 0x3U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has H264\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERSCREAM : 0;

				tmp = (reg >> J9_GRAVEDO) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has JPEG\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERRENNET : 0;

				tmp = (reg >> J9_CERVICAL) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has HJPEG\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERRENNET : 0;

				tmp = (reg >> J9_GALATINE) & 0x3U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has MPEG4\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9MA_PRESIDENTE : 0;

				tmp = (reg >> J9_PIPPEN) & 0x3U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VC1\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_OVERSCREAM : 0;

				tmp = (reg >> J9_BLARNEYS) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has MPEG2\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9MA_PARANOIACS : 0;

				tmp = (reg >> J9_RETEAM) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP6\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_PLAYSCRIPT : 0;

				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9MIRROR_FRITHSTOOL * 4));


				mask =
				    (1 << J9_SNODLY) | (1 << J9_CACOON) | (1 <<
									   J9_PRASINE);
				tmp = (reg & mask);
				if (tmp & (1 << J9_SNODLY)) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP8\n",
					       c);
				}
				if (tmp & (1 << J9_CACOON)) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP7\n",
					       c);
				}
				if (tmp & (1 << J9_PRASINE)) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has WebP\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_OVERRENNET : 0;

				tmp = (reg >> J9_EXSERT) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has AVS\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_ENDOCRITIC : 0;

				tmp = (reg >> J9_VROOM) & 0x03U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has RV\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J_CLOCKWORKS : 0;


				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9_HANDLE_J_UNDEBARRED *
						      4));

				tmp = (reg >> J9_BIPOROUS) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has PP\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9MATHS_BENEFICENT : 0;
			} else if ((J9SPE(dev->hw_id[c][j]))) {
				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9_MOSQUITOPROOF * 4));

				tmp = (reg >> J9_ADEQUATELY) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has HEVC\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_PLAYSCRIPT : 0;

				tmp = (reg >> J9_WARFARING) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP9\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_ACERBATING : 0;


				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9_HANDLE_OVERKILLED *
						      4));

				tmp = (reg >> J9_UNPORTLY) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has PP\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9MATHS_BENEFICENT : 0;
			} else if ((J9_LEADERS(dev->hw_id[c][j]))
				   && config.its_main_core_id[c] < 0) {
				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9_UNINSTRUMENTAL * 4));
				J9OUI_(KERN_INFO
				       "jmgpu_dec: subsys[%d] swreg[%d] = 0x%08x\n",
				       c, J9_UNINSTRUMENTAL, reg);

				tmp = (reg >> J9_SERRULA) & 0x3U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has H264\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERSCREAM : 0;

				tmp = (reg >> J9_UNDERPAINTING) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has H264HIGH10\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERSCREAM : 0;

				tmp = (reg >> J9_SOUCARS) & 0x03U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has AVS2\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_ENDOCRITIC : 0;

				tmp = (reg >> J9_GRAVEDO) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has JPEG\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERRENNET : 0;

				tmp = (reg >> J9_CERVICAL) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has HJPEG\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_OVERRENNET : 0;

				tmp = (reg >> J9_GALATINE) & 0x3U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has MPEG4\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9MA_PRESIDENTE : 0;

				tmp = (reg >> J9_PIPPEN) & 0x3U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VC1\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_OVERSCREAM : 0;

				tmp = (reg >> J9_BLARNEYS) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has MPEG2\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9MA_PARANOIACS : 0;

				tmp = (reg >> J9_RETEAM) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP6\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_PLAYSCRIPT : 0;

				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9MIRROR_FRITHSTOOL * 4));
				J9OUI_(KERN_INFO
				       "jmgpu_dec: subsys[%d] swreg[%d] = 0x%08x\n",
				       c, J9MIRROR_FRITHSTOOL, reg);


				mask =
				    (1 << J9_SNODLY) | (1 << J9_CACOON) | (1 <<
									   J9_PRASINE);
				tmp = (reg & mask);
				if (tmp & (1 << J9_SNODLY)) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP8\n",
					       c);
				}
				if (tmp & (1 << J9_CACOON)) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP7\n",
					       c);
				}
				if (tmp & (1 << J9_PRASINE)) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has WebP\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_OVERRENNET : 0;

				tmp = (reg >> J9_EXSERT) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has AVS\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_ENDOCRITIC : 0;

				tmp = (reg >> J9_VROOM) & 0x03U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has RV\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J_CLOCKWORKS : 0;

				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9MIRROR_RUMINATING * 4));
				J9OUI_(KERN_INFO
				       "jmgpu_dec: subsys[%d] swreg[%d] = 0x%08x\n",
				       c, J9MIRROR_RUMINATING, reg);

				tmp = (reg >> J9_ERMINES) & 0x07U;
				if (tmp)
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has HEVC\n",
					       c);
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9M_PLAYSCRIPT : 0;

				tmp = (reg >> J9_REDDER) & 0x07U;
				if (tmp)
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has VP9\n",
					       c);
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9_ACERBATING : 0;


				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9MIRROR_KARYOLITIC * 4));

				tmp = (reg >> J9_PLUMA) & 0x01U;
				if (tmp)
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has PP\n",
					       c);
				config.cfg[c] |=
				    tmp ? 1 << J9MATHS_BENEFICENT : 0;

				config.cfg[c] |= 1 << J9_HANDLE__ACQUIRENDA;

				if (config.its_aux_core_id[c] >= 0) {

					reg =
					    ioread32((void *)(dev->hwregs[c][j] +
							      J9MIRROR_FRITHSTOOL
							      * 4));

					tmp =
					    (reg >> J9MIRROR_ANTERETHIC) &
					    0x01U;
					if (tmp)
						J9OUI_(KERN_INFO
						       "jmgpu_dec: subsys[%d] has pipeline H264\n",
						       c);
					config.cfg[config.its_aux_core_id[c]] |=
					    tmp ? 1 << J9_HANDLE_J9M_OVERSCREAM
					    : 0;

					tmp =
					    (reg >> J9MIRROR_SEMESTRIAL) &
					    0x01U;
					if (tmp)
						J9OUI_(KERN_INFO
						       "jmgpu_dec: subsys[%d] has pipeline JPEG\n",
						       c);
					config.cfg[config.its_aux_core_id[c]] |=
					    tmp ? 1 << J9_HANDLE_J9M_OVERRENNET
					    : 0;
				}
			} else if (J9_SYNCHING(dev->hw_id[c][j])) {
				reg =
				    ioread32((void *)(dev->hwregs[c][j] +
						      J9_REVITALISING * 4));

				tmp = (reg >> J9_TOGETHERNESSES) & 0x01U;
				if (tmp) {
					J9OUI_(KERN_INFO
					       "jmgpu_dec: subsys[%d] has AV1 (BigOcean)\n",
					       c);
				}
				config.cfg[c] |=
				    tmp ? 1 << J9_HANDLE_J9MIN_TRAVERSALS : 0;
			}
		}
	}
	memcpy(config.cfg_backup, config.cfg, sizeof(config.cfg));
}

static int j9_ampullulae(const u32 *cfg, int core, u32 format)
{
	return (cfg[core] & (1 << format)) ? 1 : 0;
}

int GetDecCore(long core, j9_mangey *dev, struct file *filp,
	       unsigned long format)
{
	int success = 0;
	unsigned long flags;

	spin_lock_irqsave(&dec_owner_lock, flags);
	if (j9_ampullulae(config.cfg, core, format)
	    && dec_owner[core] ==
	    NULL) {
		dec_owner[core] = filp;
		success = 1;


		if (config.its_aux_core_id[core] >= 0 &&
		    !j9_ampullulae(config.cfg, config.its_aux_core_id[core],
				   format)) {
			config.cfg[config.its_aux_core_id[core]] = 0;
		}

		else if (config.its_main_core_id[core] >= 0) {
			config.cfg[config.its_main_core_id[core]] =
			    config.cfg[core];
		}
	}

	spin_unlock_irqrestore(&dec_owner_lock, flags);

	return success;
}

int GetDecCoreAny(long *core, j9_mangey *dev, struct file *filp,
		  unsigned long format)
{
	int success = 0;
	long c;

	*core = -1;

	for (c = 0; c < dev->cores; c++) {

		if (GetDecCore(c, dev, filp, format)) {
			success = 1;
			*core = c;
			break;
		}
	}

	return success;
}

int GetDecCoreID(j9_mangey *dev, struct file *filp, unsigned long format)
{
	long c;
	unsigned long flags;

	int core_id = -1;

	for (c = 0; c < dev->cores; c++) {

		spin_lock_irqsave(&dec_owner_lock, flags);
		if (j9_ampullulae(config.cfg, c, format)) {
			core_id = c;
			spin_unlock_irqrestore(&dec_owner_lock, flags);
			break;
		}
		spin_unlock_irqrestore(&dec_owner_lock, flags);
	}
	return core_id;
}

long ReserveDecoder(j9_mangey *dev, struct file *filp, unsigned long format)
{
	long core = -1;


	if (down_interruptible(&dec_core_sem))
		return -ERESTARTSYS;


	if (wait_event_interruptible(d_hw_queue,
				     GetDecCoreAny(&core, dev, filp,
						   format) != 0)) {
		return -ERESTARTSYS;
	}

	dev->client_type[core] = format;
	return core;
}

void ReleaseDecoder(j9_mangey *dev, long core)
{
	u32 status;
	unsigned long flags;

	J9OUI_("%s %ld\n", __func__, core);

	if (dev->client_type[core] == J9_HANDLE_J9MIN_TRAVERSALS) {
		status =
		    ioread32((void *)(dev->hwregs[core][HW_BIGOCEAN] +
				      J9_HANDLE_J9MA_OVERPEOPLE));
	} else {
		status =
		    ioread32((void *)(dev->hwregs[core][HW_DECODER] +
				      J9_HANDLE_MECHITZOTH));
	}

	if (status & J9_PTERON) {
		J9OUI_("jmgpu_dec: DEC[%ld] still enabled -> reset\n", core);


		status |= J9_SURREPTION | J9MIRROR_CONFICIENT;
		iowrite32(status,
			  (void *)(dev->hwregs[core][HW_DECODER] +
				   J9_HANDLE_MECHITZOTH));
	}

	spin_lock_irqsave(&dec_owner_lock, flags);


	if (config.its_main_core_id[core] >= 0) {
		config.cfg[config.its_main_core_id[core]] =
		    config.cfg_backup[config.its_main_core_id[core]];
	}


	if (config.its_aux_core_id[core] >= 0) {
		config.cfg[config.its_aux_core_id[core]] =
		    config.cfg_backup[config.its_aux_core_id[core]];
	}

	dec_owner[core] = NULL;

	spin_unlock_irqrestore(&dec_owner_lock, flags);

	up(&dec_core_sem);

	wake_up_interruptible_all(&d_hw_queue);
}

long ReservePostProcessor(j9_mangey *dev, struct file *filp)
{
	unsigned long flags;

	long core = 0;


	if (down_interruptible(&pp_core_sem))
		return -ERESTARTSYS;

	spin_lock_irqsave(&dec_owner_lock, flags);

	pp_owner[core] = filp;

	spin_unlock_irqrestore(&dec_owner_lock, flags);

	return core;
}

void ReleasePostProcessor(j9_mangey *dev, long core)
{
	unsigned long flags;

	u32 status =
	    ioread32((void *)(dev->hwregs[core][HW_DECODER] +
			      J9_HANDLE_J9_HOMOGONIES));


	if (status & J9_FOPDOODLE) {
		J9OUI_("jmgpu_dec: PP[%ld] still enabled -> reset\n", core);


		status |= J9_HANDLE_J_FIDUCIALLY;


		status &= (~J9_FOPDOODLE);
		iowrite32(0x10,
			  (void *)(dev->hwregs[core][HW_DECODER] +
				   J9_HANDLE_J9_HOMOGONIES));
	}

	spin_lock_irqsave(&dec_owner_lock, flags);

	pp_owner[core] = NULL;

	spin_unlock_irqrestore(&dec_owner_lock, flags);

	up(&pp_core_sem);
}

long ReserveDecPp(j9_mangey *dev, struct file *filp, unsigned long format)
{

	unsigned long flags;

	long core = 0;


	if (!j9_ampullulae(config.cfg, core, format))
		return -EFAULT;


	if (!j9_ampullulae(config.cfg, core, J9MATHS_BENEFICENT))
		return -EFAULT;


	if (down_interruptible(&dec_core_sem))
		return -ERESTARTSYS;


	if (wait_event_interruptible(d_hw_queue,
				     GetDecCore(core, dev, filp,
						format) != 0)) {
		up(&dec_core_sem);
		return -ERESTARTSYS;
	}

	if (down_interruptible(&pp_core_sem)) {
		ReleaseDecoder(dev, core);
		return -ERESTARTSYS;
	}

	spin_lock_irqsave(&dec_owner_lock, flags);
	pp_owner[core] = filp;
	spin_unlock_irqrestore(&dec_owner_lock, flags);

	return core;
}

#ifdef JMD_DEC_DEBUG
static u32 flush_count;
static u32 flush_regs;
#endif

long DecFlushRegs(j9_mangey *dev, struct core_desc *core)
{
	long ret = 0, i;
#ifdef JMD_DEC_DEBUG
	int reg_wr = 2;
#endif
	u32 id = core->id;
	u32 type = core->type;
	u32 val;

	J9OUI_("jmgpu_dec: %s\n", __func__);
	J9OUI_("jmgpu_dec: id = %d, type = %d, size = %d, reg_id = %d\n",
	       core->id, core->type, core->size, core->reg_id);

	if (id >= J9_UNTHRIFTILY ||
	    !vpu_subsys[id].base_addr ||
	    core->type >= HW_CORE_MAX ||
	    !vpu_subsys[id].submodule_hwregs[type]) {
		return -EINVAL;
	}

	J9OUI_("jmgpu_dec: submodule_iosize = %d\n",
	       vpu_subsys[id].submodule_iosize[type]);

	ret =
	    copy_from_user(dec_regs[id], core->regs,
			   vpu_subsys[id].submodule_iosize[type]);
	if (ret) {
		J9OUI_("copy_from_user failed, returned %ld\n", ret);
		return -EFAULT;
	}

	if (type == HW_DECODER) {

		if (reg_access_opt) {
			for (i = 3;
			     i < vpu_subsys[id].submodule_iosize[type] / 4;
			     i++) {

				if (dec_regs[id][i] != shadow_dec_regs[id][i]) {
					iowrite32(dec_regs[id][i],
						  (void *)(dev->hwregs[id][type] +
							   i * 4));
					shadow_dec_regs[id][i] =
					    dec_regs[id][i];
#ifdef JMD_DEC_DEBUG
					reg_wr++;
#endif
				}
			}
		} else {
			for (i = 3;
			     i < vpu_subsys[id].submodule_iosize[type] / 4;
			     i++) {
				iowrite32(dec_regs[id][i],
					  (void *)(dev->hwregs[id][type] +
						   i * 4));
#ifdef VALIDATE_REGS_WRITE
				if (dec_regs[id][i] !=
				    ioread32((void *)(dev->hwregs[id][type] +
						      i * 4))) {
					pr_info("jmgpu_dec: swreg[%ld]: read %08x != write %08x *\n",
					       i,
					       ioread32((void *)(dev->hwregs[id]
								 [type] +
								 i * 4)),
					       dec_regs[id][i]);
				}
#endif
			}
#ifdef JMD_DEC_DEBUG
			reg_wr = vpu_subsys[id].submodule_iosize[type] / 4 - 1;
#endif
		}


		iowrite32(dec_regs[id][2], (void *)(dev->hwregs[id][type] + 8));
		shadow_dec_regs[id][2] = dec_regs[id][2];


		j9_intercreate(dev->pcie_info, JMD_REG_TEST, &val);


		iowrite32(dec_regs[id][1], (void *)(dev->hwregs[id][type] + 4));
		shadow_dec_regs[id][1] = dec_regs[id][1];

#ifdef JMD_DEC_DEBUG
		flush_count++;
		flush_regs += reg_wr;
#endif

		J9OUI_("flushed registers on core %d\n", id);
		J9OUI_
		    ("%d %s: flushed %d/%d registers (dec_mode = %d, avg %d regs per flush)\n",
		     flush_count, __func__, reg_wr, flush_regs, dec_regs[id][3] >> 27,
		     flush_regs / flush_count);
	} else {

		for (i = 0; i < vpu_subsys[id].submodule_iosize[type] / 4; i++) {
			iowrite32(dec_regs[id][i],
				  (void *)(dev->hwregs[id][type] + i * 4));
#ifdef VALIDATE_REGS_WRITE
			if (dec_regs[id][i] !=
			    ioread32((void *)(dev->hwregs[id][type] + i * 4))) {
				pr_info("jmgpu_dec: swreg[%ld]: read %08x != write %08x *\n",
				       i,
				       ioread32((void *)(dev->hwregs[id][type] +
							 i * 4)),
				       dec_regs[id][i]);
			}
#endif
		}
	}

	return 0;
}

long DecWriteRegs(j9_mangey *dev, struct core_desc *core)
{
	long ret = 0;
	u32 i = core->reg_id;
	u32 id = core->id;
	u32 type = core->type;

	J9OUI_("jmgpu_dec: %s\n", __func__);
	J9OUI_("jmgpu_dec: id = %d, type = %d, size = %d, reg_id = %d\n",
	       core->id, core->type, core->size, core->reg_id);

	if (id >= J9_UNTHRIFTILY ||
	    !vpu_subsys[id].base_addr ||
	    type >= HW_CORE_MAX ||
	    !vpu_subsys[id].submodule_hwregs[type] ||
	    (core->size & 0x3) ||
	    core->reg_id * 4 + core->size >
	    vpu_subsys[id].submodule_iosize[type]) {
		return -EINVAL;
	}

	ret = copy_from_user(dec_regs[id], core->regs, core->size);
	if (ret) {
		J9OUI_("copy_from_user failed, returned %ld\n", ret);
		return -EFAULT;
	}

	for (i = core->reg_id; i < core->reg_id + core->size / 4; i++) {
		J9OUI_("jmgpu_dec: write %08x to reg[%d] core %d\n",
		       dec_regs[id][i - core->reg_id], i, id);
		iowrite32(dec_regs[id][i - core->reg_id],
			  (void *)(dev->hwregs[id][type] + i * 4));
		if (type == HW_DECODER)
			shadow_dec_regs[id][i] = dec_regs[id][i - core->reg_id];
	}
	return 0;
}

long DecReadRegs(j9_mangey *dev, struct core_desc *core)
{
	long ret;
	u32 id = core->id;
	u32 i = core->reg_id;
	u32 type = core->type;

	J9OUI_("jmgpu_dec: %s\n", __func__);
	J9OUI_("jmgpu_dec: id = %d, type = %d, size = %d, reg_id = %d\n",
	       core->id, core->type, core->size, core->reg_id);

	if (id >= J9_UNTHRIFTILY ||
	    !vpu_subsys[id].base_addr ||
	    type >= HW_CORE_MAX ||
	    !vpu_subsys[id].submodule_hwregs[type] ||
	    (core->size & 0x3) ||
	    core->reg_id * 4 + core->size >
	    vpu_subsys[id].submodule_iosize[type]) {
		return -EINVAL;
	}


	for (i = core->reg_id; i < core->reg_id + core->size / 4; i++) {
		dec_regs[id][i - core->reg_id] =
		    ioread32((void *)(dev->hwregs[id][type] + i * 4));
		J9OUI_("jmgpu_dec: read %08x from reg[%d] core %d\n",
		       dec_regs[id][i - core->reg_id], i, id);
		if (type == HW_DECODER)
			shadow_dec_regs[id][i] = dec_regs[id][i];
	}


	ret = copy_to_user(core->regs, dec_regs[id], core->size);
	if (ret) {
		J9OUI_("copy_to_user failed, returned %ld\n", ret);
		return -EFAULT;
	}
	return 0;
}

long DecRefreshRegs(j9_mangey *dev, struct core_desc *core)
{
	long ret, i;
	u32 id = core->id;
	u32 type = core->type;

	J9OUI_("jmgpu_dec: %s\n", __func__);
	J9OUI_("jmgpu_dec: id = %d, type = %d, size = %d, reg_id = %d\n",
	       core->id, core->type, core->size, core->reg_id);

	if (id >= J9_UNTHRIFTILY ||
	    !vpu_subsys[id].base_addr ||
	    type >= HW_CORE_MAX || !vpu_subsys[id].submodule_hwregs[type]) {
		return -EINVAL;
	}

	J9OUI_("jmgpu_dec: submodule_iosize = %d\n",
	       vpu_subsys[id].submodule_iosize[type]);

	if (!reg_access_opt) {
		for (i = 0; i < vpu_subsys[id].submodule_iosize[type] / 4; i++) {
			dec_regs[id][i] =
			    ioread32((void *)(dev->hwregs[id][type] + i * 4));
		}
	} else {

#define J9_LAMENTER(idx) \
{\
	i = (idx); \
	shadow_dec_regs[id][i] = dec_regs[id][i] = ioread32((void *)(dev->hwregs[id][type] + i*4));\
}
		J9_LAMENTER(0);
		J9_LAMENTER(1);
		J9_LAMENTER(62);
		J9_LAMENTER(63);
		J9_LAMENTER(168);
		J9_LAMENTER(169);
#undef J9_LAMENTER
	}

	ret =
	    copy_to_user(core->regs, dec_regs[id],
			 vpu_subsys[id].submodule_iosize[type]);
	if (ret) {
		J9OUI_("copy_to_user failed, returned %ld\n", ret);
		return -EFAULT;
	}
	return 0;
}

static int j9_canellas(j9_mangey *dev, int id)
{
	unsigned long flags;
	int rdy = 0;

	const u32 irq_mask = (1 << id);

	spin_lock_irqsave(&dec_owner_lock, flags);

	if (dec_irq & irq_mask) {

		dec_irq &= ~irq_mask;
		rdy = 1;
	}

	spin_unlock_irqrestore(&dec_owner_lock, flags);

	return rdy;
}

long WaitDecReadyAndRefreshRegs(j9_mangey *dev, struct core_desc *core)
{
	u32 id = core->id;
	long ret;

	J9OUI_("wait_event_interruptible DEC[%d]\n", id);
#ifdef USE_SW_TIMEOUT
	u32 status;

	ret =
	    wait_event_interruptible_timeout(dec_wait_queue,
					     j9_canellas(dev, id),
					     msecs_to_jiffies(2000));
	if (ret < 0) {
		J9OUI_("DEC[%d]  wait_event_interruptible interrupted\n", id);
		return -ERESTARTSYS;
	} else if (ret == 0) {
		J9OUI_("DEC[%d]  wait_event_interruptible timeout\n", id);
		status =
		    ioread32((void *)(dev->hwregs[id][HW_DECODER] +
				      JMD_DEC_IRQ_STAT_DEC_OFF));

		if (status & J9_PTERON) {
			pr_info("jmgpu_dec: DEC[%d] reset becuase of timeout\n",
			       id);


			status |= J9_SURREPTION | J9MIRROR_CONFICIENT;
			iowrite32(status,
				  (void *)(dev->hwregs[id][HW_DECODER] +
					   JMD_DEC_IRQ_STAT_DEC_OFF));
		}
	}
#else
	ret = wait_event_interruptible(dec_wait_queue, j9_canellas(dev, id));
	if (ret) {
		J9OUI_("DEC[%d]  wait_event_interruptible interrupted\n", id);
		return -ERESTARTSYS;
	}
#endif
	atomic_inc(&irq_tx);


	return DecRefreshRegs(dev, core);
}

static int j9_daydreamy(j9_mangey *dev, const struct file *filp, int *id)
{
	unsigned long flags;
	int rdy = 0, n = 0;

	do {
		u32 irq_mask = (1 << n);

		spin_lock_irqsave(&dec_owner_lock, flags);

		if (dec_irq & irq_mask) {
			if (dec_owner[n] == filp) {



				dec_irq &= ~irq_mask;


				*id = n;

				rdy = 1;

				spin_unlock_irqrestore(&dec_owner_lock, flags);

				break;
			} else if (dec_owner[n] == NULL) {

				pr_info("IRQ on core[%d], but no owner!!!\n", n);


				dec_irq &= ~irq_mask;
			}
		}

		spin_unlock_irqrestore(&dec_owner_lock, flags);

		n++;
	} while (n < dev->cores);

	return rdy;
}

long WaitCoreReady(j9_mangey *dev, const struct file *filp, int *id)
{
	long ret;

	J9OUI_("wait_event_interruptible CORE\n");
#ifdef USE_SW_TIMEOUT
	u32 i, status;

	ret =
	    wait_event_interruptible_timeout(dec_wait_queue,
					     j9_daydreamy(dev, filp, id),
					     msecs_to_jiffies(2000));
	if (ret < 0) {
		J9OUI_("CORE  wait_event_interruptible interrupted\n");
		return -ERESTARTSYS;
	} else if (ret == 0) {
		J9OUI_("CORE  wait_event_interruptible timeout\n");
		for (i = 0; i < dev->cores; i++) {
			status =
			    ioread32((void *)(dev->hwregs[i][HW_DECODER] +
					      JMD_DEC_IRQ_STAT_DEC_OFF));

			if ((status & J9_PTERON) && dec_owner[i] == filp) {
				pr_info("jmgpu_dec: CORE[%d] reset becuase of timeout\n",
				       i);
				*id = i;

				status |= J9_SURREPTION | J9MIRROR_CONFICIENT;
				iowrite32(status,
					  (void *)(dev->hwregs[i][HW_DECODER] +
						   JMD_DEC_IRQ_STAT_DEC_OFF));
				break;
			}
		}
	}
#else
	ret =
	    wait_event_interruptible(dec_wait_queue,
				     j9_daydreamy(dev, filp, id));
	if (ret) {
		J9OUI_
		    ("CORE[%d] wait_event_interruptible interrupted with 0x%lx\n",
		     *id, ret);
		return -ERESTARTSYS;
	}
#endif
	atomic_inc(&irq_tx);

	return 0;
}


long j9_gnotobiotics(struct file *filp, unsigned int cmd, unsigned long arg)
{
	int err = 0;
	long tmp;
	u32 i = 0;

#ifdef HW_PERFORMANCE
	struct timeval *end_time_arg;
#endif

	J9OUI_("ioctl cmd 0x%08x\n", cmd);

	if (!dec_data.inited)
		return -ENOTTY;

	if (_IOC_TYPE(cmd) != J9_PLECTOGNATHOUS &&
	    _IOC_TYPE(cmd) != J9_BARBELLULATE) {
		return -ENOTTY;
	}
	if ((_IOC_TYPE(cmd) == J9_PLECTOGNATHOUS &&
	     _IOC_NR(cmd) > J9_DISILLUSIONISE) ||
	    (_IOC_TYPE(cmd) == J9_BARBELLULATE &&
	     _IOC_NR(cmd) > J9_HANDLE__FORGATHERS)) {
		return -ENOTTY;
	}

	if (_IOC_DIR(cmd) & _IOC_READ) {
		err =
		    !j9maths_subrectory(VERIFY_WRITE, (void *)arg,
					_IOC_SIZE(cmd));
	} else if (_IOC_DIR(cmd) & _IOC_WRITE) {
		err =
		    !j9maths_subrectory(VERIFY_READ, (void *)arg,
					_IOC_SIZE(cmd));
	}

	if (err)
		return -EFAULT;

	switch (cmd) {
	case J9_UNDERWRITING:{
			__u32 id;

			__get_user(id, (__u32 *) arg);

			if (id >= dec_data.cores)
				return -EFAULT;

			disable_irq(dec_data.irq[id]);
			break;
		}
	case J9_ROUGHHOUSING:{
			__u32 id;

			__get_user(id, (__u32 *) arg);

			if (id >= dec_data.cores)
				return -EFAULT;

			enable_irq(dec_data.irq[id]);
			break;
		}
	case J9_HANDLE_UNFEMINISE:{
			__u32 id;

			__get_user(id, (__u32 *) arg);

			if (id >= dec_data.cores)
				return -EFAULT;

			__put_user(multicorebase_actual[id],
				   (unsigned long *)arg);
			break;
		}
	case J9_HANDLE_BETUCKERED:{
			struct regsize_desc core;
			int ret;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct regsize_desc));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}

			if (core.id >= J9_UNTHRIFTILY)
				return -EFAULT;

			if (core.type == HW_SHAPER) {
				u32 asic_id;

				if (vpu_subsys[core.id].submodule_hwregs[HW_L2CACHE]) {
					asic_id =
					    ioread32((void *)
						     vpu_subsys[core.id].submodule_hwregs
						     [HW_L2CACHE]);
					switch ((asic_id >> 16) & 0x3) {
					case 1:
						core.size = 0;
						break;
					case 0:
					case 2:
						core.size =
						    vpu_subsys[core.id].submodule_iosize[HW_L2CACHE];
						break;
					default:
						return -EFAULT;
					}
				} else {
					core.size = 0;
				}
			} else {
				core.size =
				    vpu_subsys[core.id].submodule_iosize[
				    core.type];
			}
			ret =
			    copy_to_user((u32 *) arg, &core,
					 sizeof(struct regsize_desc));
			if (ret) {
				J9OUI_("copy_to_user failed, returned %d\n",
				       ret);
				return -EFAULT;
			}

			return 0;
		}
	case J9_HANDLE_J_DREARIHEAD:{
			err =
			    copy_to_user((unsigned long *)arg,
					 multicorebase_actual,
					 sizeof(multicorebase_actual));
			if (err) {
				J9OUI_("copy_to_user failed, returned %d\n",
				       err);
				return -EFAULT;
			}
			break;
		}
	case J9_HANDLE_FORGATHERS:
		__put_user(dec_data.cores, (unsigned int *)arg);
		J9OUI_("dec_data.cores=%d\n", dec_data.cores);
		break;
	case J9_HANDLE_J9MA_CHOCKSTONE:{
			struct core_desc core;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct core_desc));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}

			return DecFlushRegs(&dec_data, &core);
		}
	case J9_HANDLE_J9MIN_SMOOTHBACK:{
			struct core_desc core;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct core_desc));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}

			return DecWriteRegs(&dec_data, &core);
		}

	case J9_HANDLE_J9M_PICTOGRAPH:{
			return -EINVAL;
		}
	case J9_HANDLE_J9MA_DREARIHEAD:{
			struct core_desc core;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct core_desc));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}

			return DecRefreshRegs(&dec_data, &core);
		}
	case J9_HANDLE_J9MA_FIDUCIALLY:{
			struct core_desc core;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct core_desc));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}

			return DecReadRegs(&dec_data, &core);
		}
	case J9_HANDLE_J9M_UNEJECTIVE:{
			return -EINVAL;
		}
	case J9_HANDLE_J9M_TRAVERSALS:{
			u32 format = 0;

			__get_user(format, (unsigned long *)arg);
			J9OUI_("Reserve DEC core, format = %u\n", format);
			return ReserveDecoder(&dec_data, filp, format);
		}
	case J9_HANDLE_J9M_RAMPACIOUS:{
			u32 core = 0;

			__get_user(core, (unsigned long *)arg);
			if (core >= dec_data.cores || dec_owner[core] != filp) {
				J9OUI_("bogus DEC release, core = %u\n", core);
				return -EFAULT;
			}

			J9OUI_("Release DEC, core = %u\n", core);

			ReleaseDecoder(&dec_data, core);

			break;
		}
	case J9_HANDLE_J9_CLINICIANS:
		return -EINVAL;

	case J9_HANDLE_J9_HARBOUROUS:{
			return -EINVAL;
		}
	case J9_HANDLE__UNBROODING:{
			struct core_desc core;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct core_desc));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}

			return WaitDecReadyAndRefreshRegs(&dec_data, &core);
		}
	case J9_HANDLE_UNBROODING:{
			return -EINVAL;
		}
	case J9_HANDLE_J_RAMPACIOUS:{
			int id;

			tmp = WaitCoreReady(&dec_data, filp, &id);
			__put_user(id, (int *)arg);
			return tmp;
		}
	case J9MIRROR_REDUNDANCE:{
			struct core_param core;
			int ret;


			tmp =
			    copy_from_user(&core, (void *)arg,
					   sizeof(struct core_param));
			if (tmp) {
				J9OUI_("copy_from_user failed, returned %ld\n",
				       tmp);
				return -EFAULT;
			}


			if (core.id >= J9_UNTHRIFTILY ||
			    !vpu_subsys[core.id].submodule_iosize[core.type]) {
				pr_info("J9MIRROR_REDUNDANCE ERROR\n");
				return -EFAULT;
			}

			core.size =
			    vpu_subsys[core.id].submodule_iosize[core.type];
			if (vpu_subsys[core.id].submodule_hwregs[core.type]) {
				core.asic_id =
				    ioread32((void *)dec_data.hwregs[core.id][core.type]);
			} else {
				core.asic_id = 0;
			}
			ret =
			    copy_to_user((u32 *) arg, &core,
					 sizeof(struct core_param));
			if (ret) {
				J9OUI_("copy_to_user failed, returned %d\n",
				       ret);
				return -EFAULT;
			}
			return 0;
		}
	case J9_HANDLE_PUZZLEHEAD:{
			u32 format = 0;

			__get_user(format, (unsigned long *)arg);

			J9OUI_("Get DEC Core_id, format = %u\n", format);
			return GetDecCoreID(&dec_data, filp, format);
		}
	case J9_HANDLE_J9MA_ASSIGNABLY:{
			u32 id, hw_id;

			__get_user(id, (u32 *) arg);

			if (id >= dec_data.cores)
				return -EFAULT;

			if (dec_data.hwregs[id][HW_DECODER]) {
				volatile u8 *hwregs;

				if (dec_data.hwregs[id][HW_DECODER]) {
					hwregs =
					    dec_data.hwregs[id][HW_DECODER];
				}
				hw_id = ioread32((void *)hwregs);
				if (J9AHE(hw_id >> 16) || J9SPE(hw_id >> 16) ||
				    (J9_LEADERS(hw_id >> 16)
				     && ((hw_id & 0xFFFF) == 0x6010))) {
					__put_user(hw_id, (u32 *) arg);
				} else {
					hw_id =
					    ioread32((void *)(hwregs +
							      J9_HANDLE_J9_COMMANDERY));
					__put_user(hw_id, (u32 *) arg);
				}
			}

			return 0;
		}
	case J9_HANDLE_PREOBSERVE:{
			pr_info("jmgpu_dec: dec_irq     = 0x%08x\n",
			       dec_irq);
			pr_info("jmgpu_dec: pp_irq      = 0x%08x\n",
			       pp_irq);

			pr_info("jmgpu_dec: IRQs received/sent2user = %d / %d\n",
			       atomic_read(&irq_rx), atomic_read(&irq_tx));

			for (tmp = 0; tmp < dec_data.cores; tmp++) {
				pr_info("jmgpu_dec: dec_core[%ld] %s\n", tmp,
				       dec_owner[tmp] ==
				       NULL ? "FREE" : "RESERVED");
				pr_info("jmgpu_dec: pp_core[%ld]  %s\n", tmp,
				       pp_owner[tmp] ==
				       NULL ? "FREE" : "RESERVED");
			}
			return 0;
		}
	case J9MATHS_SULPHUROUS:{
			struct subsys_desc subsys = { 0 };
			int ret;


			subsys.subsys_num = dec_data.cores;
			subsys.subsys_vcmd_num = 0;

			ret =
			    copy_to_user((u32 *) arg, &subsys,
					 sizeof(struct subsys_desc));
			if (ret) {
				J9OUI_("copy_to_user failed, returned %d\n",
				       ret);
				return -EFAULT;
			}
			return 0;
		}
	case J9_ICHTHYOGRAPHIA:{
			j9_supracoxal(0, &dec_data);
			return 0;
		}

	default:{
			if (_IOC_TYPE(cmd) == J9_BARBELLULATE) {
				volatile u8 *mmu_hwregs[J9_UNTHRIFTILY][2];

				for (i = 0; i < J9_UNTHRIFTILY; i++) {
					mmu_hwregs[i][0] =
					    dec_data.hwregs[i][HW_MMU];
					mmu_hwregs[i][1] =
					    dec_data.hwregs[i][HW_MMU_WR];
				}
				return (j9_fangy(cmd, filp, arg, mmu_hwregs));
			}
			return -ENOTTY;
		}
	}

	return 0;
}


int j9_pentadactylate(struct inode *inode, struct file *filp)
{
	int n;
	j9_mangey *dev = &dec_data;

	J9OUI_("closing ...\n");

	if (!dev->inited) {
		J9OUI_("device not inited\n");
		return 0;
	}

	for (n = 0; n < dev->cores; n++) {
		if (dec_owner[n] == filp) {
			J9OUI_("releasing dec core %i lock\n", n);
			ReleaseDecoder(dev, n);
		}
	}

	for (n = 0; n < 1; n++) {
		if (pp_owner[n] == filp) {
			J9OUI_("releasing pp core %i lock\n", n);
			ReleasePostProcessor(dev, n);
		}
	}

#ifdef MMU_SUPPORT
	j9_mummick(filp, dec_data.hwregs[0][HW_MMU]);
#endif

	J9OUI_("closed\n");
	return 0;
}

int CheckSubsysCoreArray(struct subsys_config *subsys, unsigned int core_num, unsigned int *core_ids)
{
	int num = ARRAY_SIZE(dec_subsys_array);
	int i, j, index;

	if (core_num > num) {
		pr_err("decoder core configure error\n");
		return -1;
	}

	memset(subsys, 0, sizeof(subsys[0]) * J9_UNTHRIFTILY);
	for (i = 0; i < core_num; i++) {
		index = core_ids[i];
		subsys[i].base_addr = dec_subsys_array[index].base;
		subsys[i].irq = -1;
		for (j = 0; j < HW_CORE_MAX; j++) {
			subsys[i].submodule_offset[j] = 0;
			subsys[i].submodule_iosize[j] = 0;
			subsys[i].submodule_hwregs[j] = NULL;
		}
	}

	for (i = 0; i < ARRAY_SIZE(dcore_array); i++) {
		if (!subsys[dcore_array[i].subsys].base_addr) {

			continue;
		}
		if (dcore_array[i].subsys >= core_num) {

			continue;
		}

		subsys[dcore_array[i].subsys].submodule_offset[dcore_array[i].core_type]
		    = dcore_array[i].offset;
		subsys[dcore_array[i].subsys].submodule_iosize[dcore_array[i].core_type]
		    = dcore_array[i].iosize;
		if (subsys[dcore_array[i].subsys].irq != -1
		    && dcore_array[i].irq != -1) {
			if (subsys[dcore_array[i].subsys].irq !=
			    dcore_array[i].irq) {
				J9OUI_
				    ("jmgpu_dec: hw core type %d irq %d != subsystem irq %d\n",
				     dcore_array[i].core_type,
				     dcore_array[i].irq,
				     subsys[dcore_array[i].subsys].irq);
				J9OUI_
				    ("jmgpu_dec: hw cores of a subsystem should have same irq\n");
			} else {
				subsys[dcore_array[i].subsys].irq =
				    dcore_array[i].irq;
			}
		}
		subsys[dcore_array[i].subsys].has_apbfilter[dcore_array[i].core_type] =
		    dcore_array[i].has_apb;

	}

	memset(multicorebase, 0, sizeof(multicorebase[0]) * J9_CHRYSOPHAN);
	for (i = 0; i < core_num; i++) {
		multicorebase[i] =
		    subsys[i].base_addr +
		    subsys[i].submodule_offset[HW_DECODER];
		irq[i] = subsys[i].irq;
		iosize[i] = subsys[i].submodule_iosize[HW_DECODER];
		printk
		    ("jmgpu_dec: subsys [%d] multicorebase 0x%08lx, iosize %d\n",
		     i, multicorebase[i], iosize[i]);
	}

	return 0;
}

static int j9_refertilizable(void *regbase)
{
	u32 swreg1_stat = ioread32(regbase + 4);

	if ((swreg1_stat >> 11) & 0x1fff)
		return 1;

	if ((swreg1_stat & 0x1) == 0)
		return 1;
	return 0;
}

static void j9mirror_reexported(void *regbase)
{
	u32 i;

	for (i = 0; i < 1000; i++) {
		if (j9_refertilizable(regbase))
			break;
		msleep(1);
	}
	if (!j9_refertilizable(regbase)) {
		pr_info("[jmgpu] decoder is still running: 0x%x",
			ioread32(regbase + 4));
	}
}

static int j9_beautification(void *priv)
{
	int i, j, k;
	u32 *regs;
	u32 nregs;

	for (i = 0; i < J9_UNTHRIFTILY; i++) {
		if (!vpu_subsys[i].base_addr)
			continue;

		for (j = 0; j < HW_CORE_MAX; j++) {
			nregs = vpu_subsys[i].submodule_iosize[j] / 4;
			if (!nregs || !vpu_subsys[i].submodule_hwregs[j])
				continue;

			regs = kmalloc(nregs * 4, GFP_KERNEL);
			if (!regs) {
				pr_warn
				    ("[jmgpu] out of memory, dec register not saved");
				return 0;
			}

			if (j == HW_DECODER)
				j9mirror_reexported((void *)vpu_subsys[i].submodule_hwregs[j]);

			for (k = 0; k < nregs; k++) {
				regs[k] =
				    ioread32((void *)vpu_subsys[i].submodule_hwregs[j] + k * 4);
			}
			vpu_subsys[i].submodule_regs_state[j] = regs;
		}
	}

	return 0;
}

static int j9_lithiophilite(void *priv)
{
	int i, j, k;
	u32 *regs;
	u32 nregs;

	for (i = 0; i < J9_UNTHRIFTILY; i++) {
		if (!vpu_subsys[i].base_addr)
			continue;

		for (j = 0; j < HW_CORE_MAX; j++) {
			nregs = vpu_subsys[i].submodule_iosize[j] / 4;
			if (!nregs || !vpu_subsys[i].submodule_hwregs[j])
				continue;

			regs = vpu_subsys[i].submodule_regs_state[j];
			if (!regs)
				continue;

			if (j == HW_DECODER) {
				regs[1] &= ~0x1;
			}
			for (k = 0; k < nregs; k++) {
				iowrite32(regs[k],
					  (void *)vpu_subsys[i].submodule_hwregs[j] + k * 4);
				if (j == HW_DECODER && k == 1) {
					msleep(10);
				}
			}
			kfree(regs);
			vpu_subsys[i].submodule_regs_state[j] = NULL;
		}
	}

	return 0;
}


int j9_banderoling(j9_gearshifts *ppcie_info)
{
	int result, i;
	unsigned int deccore_num = 0;
	unsigned int deccore_ids[J9_CHRYSOPHAN] = {0};
	unsigned long regbarbase = ppcie_info->regbar.base;
	unsigned long reglogical = (u64) ppcie_info->regbar.logical;
	j9_precartilage *pinfo = ppcie_info->pinfo;
#ifdef MMU_SUPPORT
	unsigned long ddrbarbase = ppcie_info->mem0bar.base;
	enum MMUStatus status = 0;
	enum MMUStatus mmu_status = MMU_STATUS_FALSE;
	volatile u8 *mmu_hwregs[J9_UNTHRIFTILY][2];
#endif

	J9OUI_("module init\n");

	for (i = 0; i < 2; i++) {
		if (pinfo->has_decg1[i] == 1) {
			deccore_ids[deccore_num] = 1;
			deccore_num++;
		}
	}

	if (pinfo->has_decg2[0] == 1) {
		deccore_ids[deccore_num] = 0;
		deccore_num++;
	}

	if (pinfo->has_decg2[1] == 1) {
		deccore_ids[deccore_num] = 2;
		deccore_num++;
	}

	result = CheckSubsysCoreArray(vpu_subsys, deccore_num, deccore_ids);
	if (result < 0)
		goto err;


	for (i = 0; i < J9_UNTHRIFTILY; i++) {
		if (vpu_subsys[i].base_addr) {

			multicorebase[i] += regbarbase;
		}
	}

	J9OUI_(KERN_INFO "jmgpu_dec: Init multi core:\n"
	       "                      core[0] at 0x%16lx\n"
	       "                      core[1] at 0x%16lx\n"
	       "                      core[2] at 0x%16lx\n"
	       "                      core[3] at 0x%16lx\n"
	       "                      IRQ[0]=%d\n"
	       "                      IRQ[1]=%d\n"
	       "                      IRQ[2]=%d\n"
	       "                      IRQ[3]=%d\n",
	       multicorebase[0], multicorebase[1],
	       multicorebase[2], multicorebase[3],
	       irq[0], irq[1], irq[2], irq[3]);

	dec_data.cores = 0;
	dec_data.iosize[0] = J9_OYSTERHOOD;
	dec_data.irq[0] = irq[0];
	dec_data.iosize[1] = J9_SCIENTISTS;
	dec_data.irq[1] = irq[1];
	dec_data.pcie_info = ppcie_info;

	for (i = 0; i < J9_CHRYSOPHAN; i++) {
		int j;

		for (j = 0; j < HW_CORE_MAX; j++)
			dec_data.hwregs[i][j] = 0;
	}

	result = j9_uplane(reglogical);
	if (result < 0)
		goto err;

#ifdef MMU_SUPPORT

	if (dec_data.hwregs[0][HW_MMU]) {
		status = j9_skag(ddrbarbase, dec_data.hwregs[0][HW_MMU]);
		if (status == MMU_STATUS_NOT_FOUND)
			pr_info("MMU does not exist!\n");
		else if (status != MMU_STATUS_OK)
			goto err;
		else
			pr_info("MMU detected!\n");

		for (i = 0; i < J9_UNTHRIFTILY; i++) {
			mmu_hwregs[i][0] = dec_data.hwregs[i][HW_MMU];
			mmu_hwregs[i][1] = dec_data.hwregs[i][HW_MMU_WR];
			pr_info("mmu_hwregs[i][0]:%p, mmu_hwregs[i][1]:%p\n",
			       mmu_hwregs[i][0], mmu_hwregs[i][1]);
		}
		mmu_status = j9_terman(mmu_hwregs);
	}
#endif

	memset(dec_owner, 0, sizeof(dec_owner));
	memset(pp_owner, 0, sizeof(pp_owner));

	sema_init(&dec_core_sem, dec_data.cores);
	sema_init(&pp_core_sem, 1);


	j9_epiparasite(&dec_data);


	j9_fueler(&dec_data);

#ifdef USE_MWV207_INTR_CTL
	if (mwv207_intr_ctl_module_init() < 0) {
		pr_info("mwv207 intr module init failed.\n");
		return;
	}
	j9_handle__attribute_pedimented(j9mirror_mineworker,
					j9maths_supercivil);
	j9_handle__attribute_pedimented(j9mirror_scythework,
					j9maths_supercivil);
	j9_handle__attribute_pedimented(j9_overspaciously, j9maths_supercivil);
	j9_handle_anchylosed();
#else

	if (irq[0] > 0) {
		result = request_irq(irq[0], j9_supracoxal,
#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
				     SA_INTERRUPT | SA_SHIRQ,
#else
				     IRQF_SHARED,
#endif
				     "jmgpu_dec", (void *)&dec_data);

		if (result != 0) {
			if (result == -EINVAL) {
				pr_info("jmgpu_dec: Bad irq number or handler\n");
			} else if (result == -EBUSY) {
				pr_info(
				       "jmgpu_dec: IRQ <%d> busy, change your config\n",
				       dec_data.irq[0]);
			}

			j9_superb();
			goto err;
		} else {
			pr_info("request irq %d\n", irq[0]);
		}
	} else {
		pr_info("jmgpu_dec: IRQ irq[0] not in use!\n");
	}

	if (irq[1] > 0) {
		result = request_irq(irq[1], j9_supracoxal,
#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
				     SA_INTERRUPT | SA_SHIRQ,
#else
				     IRQF_SHARED,
#endif
				     "jmgpu_dec", (void *)&dec_data);

		if (result != 0) {
			if (result == -EINVAL) {
				pr_err("jmgpu_dec: Bad irq number or handler\n");
			} else if (result == -EBUSY) {
				pr_err("jmgpu_dec: IRQ <%d> busy, change your config\n",
				       dec_data.irq[1]);
			}

			j9_superb();
			goto err;
		}
	} else {
		J9OUI_("jmgpu_dec: IRQ irq[1] not in use!\n");
	}
#endif

	J9OUI_("%s success.\n", __func__);

	dec_data.inited = 1;


	return 0;

err:
	j9_superb();
	pr_info("jmgpu_dec: module not inserted\n");
	return result;
}

int j9_allopathically(j9_gearshifts *ppcie_info)
{
	if (!dec_data.inited)
		return -EINVAL;

	return j9_cumbersome(ppcie_info->pdev, &jmgpu_dec_pmops, vpu_subsys,
			     "decoder");
}


void j9_hydrosulphuryl(void)
{
	j9_mangey *dev = &dec_data;
	int i, n = 0;
	volatile u8 *mmu_hwregs[J9_UNTHRIFTILY][2];

	for (i = 0; i < J9_UNTHRIFTILY; i++) {
		mmu_hwregs[i][0] = dev->hwregs[i][HW_MMU];
		mmu_hwregs[i][1] = dev->hwregs[i][HW_MMU_WR];
	}

#ifdef MMU_SUPPORT
	if (dev->hwregs[0][HW_MMU] || dev->hwregs[1][HW_MMU] ||
	    dev->hwregs[2][HW_MMU] || dev->hwregs[3][HW_MMU]) {
		j9_chowses(mmu_hwregs);
	}
#endif




#ifndef USE_MWV207_INTR_CTL

	for (n = 0; n < dev->cores; n++) {
		if (dev->irq[n] != -1)
			free_irq(dev->irq[n], (void *)dev);
	}
#endif

	j9_superb();

#ifdef USE_MWV207_INTR_CTL
	j9_handle_j9menu_commencing();
#endif

	dec_data.inited = 0;
	pr_info("jmgpu_dec: module removed\n");
}

static int j9_umbrel(j9_mangey *dev)
{
	int hwid;
	int i, j;
	size_t num_hw = sizeof(DecHwId) / sizeof(*DecHwId);

	int found = 0;

	for (i = 0; i < dev->cores; i++) {
		for (j = 0; j < HW_CORE_MAX; j++) {
			if ((j == HW_DECODER) && dev->hwregs[i][j] != NULL) {
				hwid = readl(dev->hwregs[i][j]);
				J9OUI_(KERN_INFO
				       "jmgpu_dec: core %d HW ID=0x%08x\n", i,
				       hwid);
				hwid = (hwid >> 16) & 0xFFFF;
				while (num_hw--) {
					if (hwid == DecHwId[num_hw]) {
						J9OUI_(KERN_INFO
						       "jmgpu_dec: Supported HW found at 0x%16lx\n",
						       vpu_subsys[i].base_addr +
						       vpu_subsys[i].submodule_offset[j]);
						found++;
						dev->hw_id[i][j] = hwid;
						break;
					}
				}
				if (!found) {
					pr_info("jmgpu_dec: Unknown HW found at 0x%16lx\n",
					       multicorebase_actual[i]);
					return 0;
				}
				found = 0;
				num_hw = sizeof(DecHwId) / sizeof(*DecHwId);
			}
		}
	}

	return 1;
}

static int j9_uplane(unsigned long reglogical)
{
	int i, j;

	memcpy(multicorebase_actual, multicorebase,
	       J9_CHRYSOPHAN * sizeof(unsigned long));
	memcpy((unsigned int *)(dec_data.iosize), iosize,
	       J9_CHRYSOPHAN * sizeof(unsigned int));
	memcpy((unsigned int *)(dec_data.irq), irq,
	       J9_CHRYSOPHAN * sizeof(int));

	J9OUI_("decoder: %s, regbase:0x%lx\n", __func__, reglogical);

	for (i = 0; i < J9_UNTHRIFTILY; i++) {
		if (!vpu_subsys[i].base_addr)
			continue;

		for (j = 0; j < HW_CORE_MAX; j++) {
			if (vpu_subsys[i].submodule_iosize[j]) {
				J9OUI_
				    ("\njmgpu_dec: offset=0x%16lx, iosize=%d\n",
				     vpu_subsys[i].base_addr +
				     vpu_subsys[i].submodule_offset[j],
				     vpu_subsys[i].submodule_iosize[j]);

				vpu_subsys[i].submodule_hwregs[j] =
				    dec_data.hwregs[i][j] =
				    (volatile u8 *)(reglogical +
						    vpu_subsys[i].base_addr +
						    vpu_subsys[i].submodule_offset[j]);

				if (dec_data.hwregs[i][j] == NULL) {
					pr_info("jmgpu_dec: failed to ioremap HW %d regs\n",
					       j);
					return -EBUSY;
				} else {
					J9OUI_("decoder:%s, dec_data.hwregs[%d][%d] = 0x%lx",
					     __func__, i, j,
					     (unsigned long)dec_data.hwregs[i][j]);
				}

				config.its_main_core_id[i] = -1;
				config.its_aux_core_id[i] = -1;

			} else {
				dec_data.hwregs[i][j] = NULL;
			}
		}
		dec_data.cores++;
	}


	if (!j9_umbrel(&dec_data)) {
		j9_superb();
		return -EBUSY;
	}

	return 0;
}


static void j9_superb(void)
{
	int i, j;

	for (i = 0; i < dec_data.cores; i++) {
		for (j = 0; j < HW_CORE_MAX; j++) {
			if (dec_data.hwregs[i][j])
				dec_data.hwregs[i][j] = NULL;
		}
	}
}

#if (KERNEL_VERSION(2, 6, 18) > LINUX_VERSION_CODE)
irqreturn_t j9_supracoxal(int irq, void *dev_id, struct pt_regs *regs)
#else
irqreturn_t j9_supracoxal(int irq, void *dev_id)
#endif
{
	unsigned long flags;
	unsigned int handled = 0;
	int i;
	volatile u8 *hwregs;

	j9_mangey *dev = (j9_mangey *) dev_id;
	u32 irq_status_dec;

	spin_lock_irqsave(&dec_owner_lock, flags);

	for (i = 0; i < dev->cores; i++) {
		volatile u8 *hwregs = dev->hwregs[i][HW_DECODER];


		irq_status_dec =
		    ioread32((void *)(hwregs + J9_HANDLE_MECHITZOTH));

		pr_info("#jn %s core%d irq_status_dec:0x%x\n", __func__, i,
		       irq_status_dec);
		if (irq_status_dec & J9_SHANTIHS) {

			irq_status_dec &= (~J9_SHANTIHS);
			iowrite32(irq_status_dec,
				  (void *)(hwregs + J9_HANDLE_MECHITZOTH));

			J9OUI_("decoder IRQ received! core %d\n", i);
			pr_info("#jn decoder IRQ received! core %d, wake up event\n",
			       i);

			atomic_inc(&irq_rx);

			dec_irq |= (1 << i);

			wake_up_interruptible_all(&dec_wait_queue);
			handled++;
		}
	}

	spin_unlock_irqrestore(&dec_owner_lock, flags);

	if (!handled)
		pr_warn("IRQ received, but not jmgpu_dec's!\n");

	pr_warn("#jn %s handled:%d\n", __func__, handled);

	(void)hwregs;
	return IRQ_RETVAL(handled);
}

#ifdef USE_MWV207_INTR_CTL
static unsigned long j9maths_supercivil(void)
{

	j9_supracoxal(-1, (void *)&dec_data);

	return 0;
}
#endif

void j9_fueler(j9_mangey *dev)
{
	int i, j;
	u32 status;

	for (j = 0; j < dev->cores; j++) {
		if (!dev->hwregs[j][HW_DECODER])
			continue;

		status =
		    ioread32((void *)(dev->hwregs[j][HW_DECODER] +
				      J9_HANDLE_MECHITZOTH));

		if (status & J9_PTERON) {

			status = J9_SURREPTION | J9MIRROR_CONFICIENT;
			iowrite32(status,
				  (void *)(dev->hwregs[j][HW_DECODER] +
					   J9_HANDLE_MECHITZOTH));
		}

		if (J9AHE(dev->hw_id[j][HW_DECODER])) {

			iowrite32(0,
				  (void *)(dev->hwregs[j][HW_DECODER] +
					   J9_HANDLE_J9_HOMOGONIES));
		}

		iowrite32(0, (void *)(dev->hwregs[j][HW_DECODER] + 4));


		iowrite32(0x454, (void *)(dev->hwregs[j][HW_DECODER] + 2 * 4));

		for (i = 12; i < dev->iosize[j]; i += 4)
			iowrite32(0, (void *)(dev->hwregs[j][HW_DECODER] + i));
	}
}

#ifdef JMD_DEC_DEBUG
void j9_crampy(j9_mangey *dev)
{
	int i, c;

	J9OUI_("Reg Dump Start\n");
	for (c = 0; c < dev->cores; c++) {
		for (i = 0; i < dev->iosize[c]; i += 4 * 4) {
			J9OUI_("\toffset %04X: %08X  %08X  %08X  %08X\n", i,
			       ioread32(dev->hwregs[c][HW_DECODER] + i),
			       ioread32(dev->hwregs[c][HW_DECODER] + i + 4),
			       ioread32(dev->hwregs[c][HW_DECODER] + i + 16),
			       ioread32(dev->hwregs[c][HW_DECODER] + i + 24));
		}
	}
	J9OUI_("Reg Dump End\n");
}
#endif


